import os
import sys
import argparse
from pathlib import Path
from rwc import __version__
from rwc.render import can_render, wordcount


def argument_parser():
    parser = argparse.ArgumentParser(description='reStructuredText and Markdown wordcounter')
    parser.add_argument('--version', action='version', version=f"%(prog)s {__version__}")
    parser.add_argument('paths', metavar='PATH', nargs='+', help='paths to count')
    return parser 


def main():
    parser = argument_parser()
    args = parser.parse_args()

    # for each path given, check if it's a file or a directory
    # if it's a directory, get a list of all files in that directory
    paths = []
    for path in args.paths:
        path = Path(path)
        if path.is_file():
            paths.append(path)

        elif path.is_dir():
            for subpath in path.glob('**/*'):
                if subpath.is_file():
                    paths.append(subpath)

    # perform counts
    counts = {}
    for path in paths:
        if can_render(path):
            with open(path, 'r') as fh:
                count = wordcount(str(path), fh.read())
                if count is not None:
                    counts[str(path)] = count

    # print the per-file counts
    for path in counts.keys():
        print(f"{path}: {counts[path]} words")

    # and print the total
    print("-----")
    print(f"total: {sum(counts.values())} words")

    return 0



RENDER_FUNCTIONS = {}


def render_func(*extensions):
    """Register a function as an HTML renderer for the given file
    extensions.
    """

    def inner(function):
        global RENDER_FUNCTIONS
        for ext in extensions:
            RENDER_FUNCTIONS[ext] = function

        return function
    return inner


from . import restructuredtext
from . import markdown
from . import html



from . import render_func
from docutils.core import publish_parts


@render_func('.rst')
def render_rst(content):
    """Render the reStructuredText ``content`` to HTML.
    """

    parts = publish_parts(content, writer_name='html')
    return parts["html_body"]



from .formats import RENDER_FUNCTIONS
from html_sanitizer import Sanitizer
from pathlib import Path
import re


def sanitize(content):
    """Get just the text from the HTML ``content``.
    """

    sanitizer = Sanitizer({
        'tags': set('this-tag-will-never-exist-in-html'),
        'attributes': {},
        'empty': set(),
        'separate': set(),
    })

    return sanitizer.sanitize(content)


def can_render(filename):
    ext = Path(filename).suffix
    renderer = RENDER_FUNCTIONS.get(ext, None)
    return renderer is not None


def render(filename, content):
    """Render the ``content`` as HTML, using the renderer from
    the file extension from ``filename``.
    """

    ext = Path(filename).suffix
    renderer = RENDER_FUNCTIONS.get(ext, None)
    if renderer is None:
        return None

    output = renderer(content)
    output = sanitize(output)
    return output


def wordcount(filename, content):
    """Get the word count for the ``content``, using the renderer from
    the file extension from ``filename``.
    """

    output = render(filename, content)
    if output is None:
        return None

    match = re.findall(r"\w+", output, re.MULTILINE)
    return len(match)



from . import render_func
import markdown


@render_func('.md', '.mkd', '.markdown')
def render_markdown(content):
    """Render the Markdown ``content`` to HTML.
    """

    return markdown.markdown(content)
